#ifndef FPUdefs
#define FPUdefs

'==============================================================================
' Copyright  Micromega Corporation 2002-2007. All rights reserved.
'
' @file		FPUdefs.bas
' @target:	ARMmite/ARMexpress
'
' This file contains definitions for uM-FPU V3 instructions and operations.
' Definitions are compatible with both V3.0 and V3.1, but some instructions
' and operations are only available on V3.1. See the uM-FPU V3.1 Instruction Set
' for details.
'
' @author Cam Thompson, Micromega Corporation
' @version
'	August 2, 2007
'	- original version
'==============================================================================

'-------------------- uM-FPU V3.1 opcode definitions --------------------------

const	NOP			= $00	' No Operation
const	SELECTA		= $01	' Select register A
const	SELECTX		= $02	' Select register X
const	CLR			= $03	' reg[nn] = 0
const	CLRA		= $04	' reg[A] = 0
const	CLRX		= $05	' reg[X] = 0, X = X + 1
const	CLR0		= $06	' reg[0] = 0
const	COPY		= $07	' reg[nn] = reg[mm]
const	COPYA		= $08	' reg[nn] = reg[A]
const	COPYX		= $09	' reg[nn] = reg[X], X = X + 1
const	LOAD		= $0A	' reg[0] = reg[nn]
const	LOADA		= $0B	' reg[0] = reg[A]
const	LOADX		= $0C	' reg[0] = reg[X], X = X + 1
const	ALOADX		= $0D	' reg[A] = reg[X], X = X + 1
const	XSAVE		= $0E	' reg[X] = reg[nn], X = X + 1
const	XSAVEA		= $0F	' reg[X] = reg[A], X = X + 1
const	COPY0		= $10	' reg[nn] = reg[0]
const	COPYI		= $11	' reg[nn] = long(unsigned bb)
const	SWAP		= $12	' Swap reg[nn] and reg[mm]
const	SWAPA		= $13	' Swap reg[A] and reg[nn]
const	F_LEFT		= $14	' Left parenthesis
const	F_RIGHT		= $15	' Right parenthesis
const	FWRITE		= $16	' Write 32-bit float to reg[nn]
const	FWRITEA		= $17	' Write 32-bit float to reg[A]
const	FWRITEX		= $18	' Write 32-bit float to reg[X], X = X + 1
const	FWRITE0		= $19	' Write 32-bit float to reg[0]
const	F_FREAD		= $1A	' Read 32-bit float from reg[nn]
const	FREADA		= $1B	' Read 32-bit float from reg[A]
const	FREADX		= $1C	' Read 32-bit float from reg[X], X = X + 1
const	FREAD0		= $1D	' Read 32-bit float from reg[0]
const	ATOF		= $1E	' Convert ASCII to float, store in reg[0]
const	FTOA		= $1F	' Convert float to ASCII
const	FSET		= $20	' reg[A] = reg[nn]
const	FADD		= $21	' reg[A] = reg[A] + reg[nn]
const	FSUB		= $22	' reg[A] = reg[A] - reg[nn]
const	FSUBR		= $23	' reg[A] = reg[nn] - reg[A]
const	FMUL		= $24	' reg[A] = reg[A] * reg[nn]
const	FDIV		= $25	' reg[A] = reg[A] / reg[nn]
const	FDIVR		= $26	' reg[A] = reg[nn] / reg[A]
const	FPOW		= $27	' reg[A] = reg[A] ** reg[nn]
const	FCMP		= $28	' Float compare reg[A] - reg[nn]
const	FSET0		= $29	' reg[A] = reg[0]
const	FADD0		= $2A	' reg[A] = reg[A] + reg[0]
const	FSUB0		= $2B	' reg[A] = reg[A] - reg[0]
const	FSUBR0		= $2C	' reg[A] = reg[0] - reg[A]
const	FMUL0		= $2D	' reg[A] = reg[A] * reg[0]
const	FDIV0		= $2E	' reg[A] = reg[A] / reg[0]
const	FDIVR0		= $2F	' reg[A] = reg[0] / reg[A]
const	FPOW0		= $30	' reg[A] = reg[A] ** reg[0]
const	FCMP0		= $31	' Float compare reg[A] - reg[0]
const	FSETI		= $32	' reg[A] = float(bb)
const	FADDI		= $33	' reg[A] = reg[A] + float(bb)
const	FSUBI		= $34	' reg[A] = reg[A] - float(bb)
const	FSUBRI		= $35	' reg[A] = float(bb) - reg[A]
const	FMULI		= $36	' reg[A] = reg[A] * float(bb)
const	FDIVI		= $37	' reg[A] = reg[A] / float(bb)
const	FDIVRI		= $38	' reg[A] = float(bb) / reg[A]
const	FPOWI		= $39	' reg[A] = reg[A] ** bb
const	FCMPI		= $3A	' Float compare reg[A] - float(bb)
const	FSTATUS		= $3B	' Float status of reg[nn]
const	FSTATUSA	= $3C	' Float status of reg[A]
const	FCMP2		= $3D	' Float compare reg[nn] - reg[mm]
const	FNEG		= $3E	' reg[A] = -reg[A]
const	FABS		= $3F	' reg[A] = | reg[A] |
const	FINV		= $40	' reg[A] = 1 / reg[A]
const	SQRT		= $41	' reg[A] = sqrt(reg[A])
const	ROOT		= $42	' reg[A] = root(reg[A], reg[nn])
const	LOG			= $43	' reg[A] = log(reg[A])
const	LOG10		= $44	' reg[A] = log10(reg[A])
const	EXP			= $45	' reg[A] = exp(reg[A])
const	EXP10		= $46	' reg[A] = exp10(reg[A])
const	F_SIN		= $47	' reg[A] = sin(reg[A])
const	F_COS		= $48	' reg[A] = cos(reg[A])
const	TAN			= $49	' reg[A] = tan(reg[A])
const	ASIN		= $4A	' reg[A] = asin(reg[A])
const	ACOS		= $4B	' reg[A] = acos(reg[A])
const	ATAN		= $4C	' reg[A] = atan(reg[A])
const	ATAN2		= $4D	' reg[A] = atan2(reg[A], reg[nn])
const	DEGREES		= $4E	' reg[A] = degrees(reg[A])
const	RADIANS		= $4F	' reg[A] = radians(reg[A])
const	FMOD		= $50	' reg[A] = reg[A] MOD reg[nn]
const	FLOOR		= $51	' reg[A] = floor(reg[A])
const	CEIL		= $52	' reg[A] = ceil(reg[A])
const	ROUND		= $53	' reg[A] = round(reg[A])
const	FMIN		= $54	' reg[A] = min(reg[A], reg[nn])
const	FMAX		= $55	' reg[A] = max(reg[A], reg[nn])
const	FCNV		= $56	' reg[A] = conversion(nn, reg[A])
const	FMAC		= $57	' reg[A] = reg[A] + (reg[nn] * reg[mm])
const	FMSC		= $58	' reg[A] = reg[A] - (reg[nn] * reg[mm])
const	LOADBYTE	= $59	' reg[0] = float(signed bb)
const	LOADUBYTE	= $5A	' reg[0] = float(unsigned byte)
const	LOADWORD	= $5B	' reg[0] = float(signed word)
const	LOADUWORD	= $5C	' reg[0] = float(unsigned word)
const	LOADE		= $5D	' reg[0] = 2.7182818
const	LOADPI		= $5E	' reg[0] = 3.1415927
const	LOADCON		= $5F	' reg[0] = float constant(nn)
const	FLOAT		= $60	' reg[A] = float(reg[A])
const	FIX			= $61	' reg[A] = fix(reg[A])
const	FIXR		= $62	' reg[A] = fix(round(reg[A]))
const	FRAC		= $63	' reg[A] = fraction(reg[A])
const	FSPLIT		= $64	' reg[A] = int(reg[A]), reg[0] = frac(reg[A])
const	SELECTMA	= $65	' Select matrix A
const	SELECTMB	= $66	' Select matrix B
const	SELECTMC	= $67	' Select matrix C
const	LOADMA		= $68	' reg[0] = matrix A[bb,bb]
const	LOADMB		= $69	' reg[0] = matrix B[bb, bb]
const	LOADMC		= $6A	' reg[0] = matrix C[bb, bb]
const	SAVEMA		= $6B	' matrix A[bb,bb] = reg[A]
const	SAVEMB		= $6C	' matrix B[bb,bb] = reg[A]
const	SAVEMC		= $6D	' matrix C[bb,bb] = reg[A]
const	MOP			= $6E	' Matrix operation
const	FFT			= $6F	' FFT operation
const	WRBLK		= $70	' write register block
const	RDBLK		= $71	' read register block
const	LOADIND		= $7A	' reg[0] = reg[reg[nn]]
const	SAVEIND		= $7B	' reg[reg[nn]] = reg[A]
const	INDA		= $7C	' Select A using reg[nn]
const	INDX		= $7D	' Select X using reg[nn]
const	FCALL		= $7E	' Call function in Flash memory
const	EECALL		= $7F	' Call function in EEPROM memory
const	RET			= $80	' Return from function
const	BRA			= $81	' Unconditional branch
const	BRACC		= $82	' Conditional branch
const	JMP			= $83	' Unconditional jump
const	JMPCC		= $84	' Conditional jump
const	TABLE		= $85	' Table lookup
const	FTABLE		= $86	' Floating point reverse table lookup
const	LTABLE		= $87	' Long integer reverse table lookup
const	POLY		= $88	' reg[A] = nth order polynomial
const	F_GOTO		= $89	' Computed goto
const	RETCC		= $8A	' Conditional return from function
const	LWRITE		= $90	' Write 32-bit long integer to reg[nn]
const	LWRITEA		= $91	' Write 32-bit long integer to reg[A]
const	LWRITEX		= $92	' Write 32-bit long integer to reg[X], X = X + 1
const	LWRITE0		= $93	' Write 32-bit long integer to reg[0]
const	LREAD		= $94	' Read 32-bit long integer from reg[nn]
const	LREADA		= $95	' Read 32-bit long integer from reg[A]
const	LREADX		= $96	' Read 32-bit long integer from reg[X], X = X + 1
const	LREAD0		= $97	' Read 32-bit long integer from reg[0]
const	LREADBYTE	= $98	' Read lower 8 bits of reg[A]
const	LREADWORD	= $99	' Read lower 16 bits reg[A]
const	ATOL		= $9A	' Convert ASCII to long integer
const	LTOA		= $9B	' Convert long integer to ASCII
const	LSET		= $9C	' reg[A] = reg[nn]
const	LADD		= $9D	' reg[A] = reg[A] + reg[nn]
const	LSUB		= $9E	' reg[A] = reg[A] - reg[nn]
const	LMUL		= $9F	' reg[A] = reg[A] * reg[nn]
const	LDIV		= $A0	' reg[A] = reg[A] / reg[nn]
const	LCMP		= $A1	' Signed long compare reg[A] - reg[nn]
const	LUDIV		= $A2	' reg[A] = reg[A] / reg[nn]
const	LUCMP		= $A3	' Unsigned long compare of reg[A] - reg[nn]
const	LTST		= $A4	' Long integer status of reg[A] AND reg[nn] 
const	LSET0		= $A5	' reg[A] = reg[0]
const	LADD0		= $A6	' reg[A] = reg[A] + reg[0]
const	LSUB0		= $A7	' reg[A] = reg[A] - reg[0]
const	LMUL0		= $A8	' reg[A] = reg[A] * reg[0]
const	LDIV0		= $A9	' reg[A] = reg[A] / reg[0]
const	LCMP0		= $AA	' Signed long compare reg[A] - reg[0]
const	LUDIV0		= $AB	' reg[A] = reg[A] / reg[0]
const	LUCMP0		= $AC	' Unsigned long compare reg[A] - reg[0]
const	LTST0		= $AD	' Long integer status of reg[A] AND reg[0] 
const	LSETI		= $AE	' reg[A] = long(bb)
const	LADDI		= $AF	' reg[A] = reg[A] + long(bb)
const	LSUBI		= $B0	' reg[A] = reg[A] - long(bb)
const	LMULI		= $B1	' reg[A] = reg[A] * long(bb)
const	LDIVI		= $B2	' reg[A] = reg[A] / long(bb)
const	LCMPI		= $B3	' Signed long compare reg[A] - long(bb)
const	LUDIVI		= $B4	' reg[A] = reg[A] / unsigned long(bb)
const	LUCMPI		= $B5	' Unsigned long compare reg[A] - ulong(bb)
const	LTSTI		= $B6	' Long integer status of reg[A] AND ulong(bb)
const	LSTATUS		= $B7	' Long integer status of reg[nn]
const	LSTATUSA	= $B8	' Long integer status of reg[A]
const	LCMP2		= $B9	' Signed long compare reg[nn] - reg[mm]
const	LUCMP2		= $BA	' Unsigned long compare reg[nn] - reg[mm]
const	LNEG		= $BB	' reg[A] = -reg[A]
const	LABS		= $BC	' reg[A] = | reg[A] |
const	LINC		= $BD	' reg[nn] = reg[nn] + 1
const	LDEC		= $BE	' reg[nn] = reg[nn] - 1
const	LNOT		= $BF	' reg[A] = NOT reg[A]
const	LAND		= $C0	' reg[A] = reg[A] AND reg[nn]
const	LOR			= $C1	' reg[A] = reg[A] OR reg[nn]
const	LXOR		= $C2	' reg[A] = reg[A] XOR reg[nn]
const	LSHIFT		= $C3	' reg[A] = reg[A] shift reg[nn]
const	LMIN		= $C4	' reg[A] = min(reg[A], reg[nn])
const	LMAX		= $C5	' reg[A] = max(reg[A], reg[nn])
const	LONGBYTE	= $C6	' reg[0] = long(signed byte bb)
const	LONGUBYTE	= $C7	' reg[0] = long(unsigned byte bb)
const	LONGWORD	= $C8	' reg[0] = long(signed word wwww)
const	LONGUWORD	= $C9	' reg[0] = long(unsigned word wwww)
const	SETSTATUS	= $CD	' Set status byte
const	F_SEROUT	= $CE	' Serial output
const	F_SERIN		= $CF	' Serial Input
const	SETOUT		= $D0	' Set OUT1 and OUT2 output pins
const	ADCMODE		= $D1	' Set A/D trigger mode
const	ADCTRIG		= $D2	' A/D manual trigger
const	ADCSCALE	= $D3	' ADCscale[ch] = B
const	ADCLONG		= $D4	' reg[0] = ADCvalue[ch]
const	ADCLOAD		= $D5	' reg[0] = float(ADCvalue[ch]) * ADCscale[ch]
const	ADCWAIT		= $D6	' wait for next A/D sample
const	TIMESET		= $D7	' time = reg[0]
const	TIMELONG	= $D8	' reg[0] = time (long)
const	TICKLONG	= $D9	' reg[0] = ticks (long)
const	EESAVE		= $DA	' EEPROM[nn] = reg[mm]
const	EESAVEA		= $DB	' EEPROM[nn] = reg[A]
const	EELOAD		= $DC	' reg[nn] = EEPROM[mm]
const	EELOADA		= $DD	' reg[A] = EEPROM[nn]
const	EEWRITE		= $DE	' Store bytes in EEPROM
const	EXTSET		= $E0	' external input count = reg[0]
const	EXTLONG		= $E1	' reg[0] = external input counter (long)
const	EXTWAIT		= $E2	' wait for next external input
const	STRSET		= $E3	' Copy string to string buffer
const	STRSEL		= $E4	' Set selection point
const	STRINS		= $E5	' Insert string at selection point
const	STRCMP		= $E6	' Compare string with string buffer
const	STRFIND		= $E7	' Find string and set selection point
const	STRFCHR		= $E8	' Set field separators
const	STRFIELD	= $E9	' Find field and set selection point
const	STRTOF		= $EA	' Convert string selection to float
const	STRTOL		= $EB	' Convert string selection to long
const	READSEL		= $EC	' Read string selection
const	STRBYTE		= $ED	' Insert 8-bit byte at selection point
const	STRINC		= $EE	' increment selection point
const	STRDEC		= $EF	' decrement selection point
const	SYNC		= $F0	' Get synchronization byte
const	READSTATUS	= $F1	' Read status byte
const	READSTR		= $F2	' Read string from string buffer
const	VERSION		= $F3	' Copy version string to string buffer
const	IEEEMODE	= $F4	' Set IEEE mode (default)
const	PICMODE		= $F5	' Set PIC mode
const	CHECKSUM	= $F6	' Calculate checksum for uM-FPU code
const	BREAK		= $F7	' Debug breakpoint
const	TRACEOFF	= $F8	' Turn debug trace off
const	TRACEON		= $F9	' Turn debug trace on
const	TRACESTR	= $FA	' Send string to debug trace buffer
const	TRACEREG	= $FB	' Send register value to trace buffer
const	READVAR		= $FC	' Read internal variable, store in reg[0]
const	RESET		= $FF	' No operation (9 consecutive FF bytes cause a reset)

const	SYNC_CHAR	= $5C	' sync character

const	STATUS_ZERO	= $01   ' zero status bit
const	STATUS_SIGN = $02	' sign status bit (set if negative)
const	STATUS_NAN	= $04	' Not-a-Number bit
const	STATUS_INF	= $08	' infinity status bit

const	STR_MAX		= 128	' string buffer (20 to 128 bytes)
dim		str$(STR_MAX)

#endif

#define FPUdefs				' Note: remove this line to define extra symbols

#ifndef FPUdefs
#define FPUdefs

'-------------------- uM-FPU V3.1 MOP definitions -----------------------------


const	SCALAR_SET	= 0		' MA[row,col] = reg[0], for all elements
const	SCALAR_ADD	= 1		' MA[row,col] = MA[row,col] + reg[0], for all elements
const	SCALAR_SUB	= 2		' MA[row,col] = reg[0] - MA[row,col], for all elements
const	SCALAR_SUBR	= 3		' MA[row,col] = MA[row,col] + reg[0], for all elements
const	SCALAR_MUL	= 4		' MA[row,col] = MA[row,col] * reg[0], for all elements
const	SCALAR_DIV	= 5		' MA[row,col] = MA[row,col] / reg[0], for all elements
const	SCALAR_DIVR	= 6		' MA[row,col] = reg[0] / MA[row,col], for all elements
const	SCALAR_POW	= 7		' MA[row,col] = MA[row,col] ** reg[0], for all elements
const	EWISE_SET	= 8		' MA[row,col] = MB[row,col], for all elements
const	EWISE_ADD	= 9		' MA[row,col] = MA[row,col] + MB[row,col], for all elements
const	EWISE_SUB	= 10	' MA[row,col] = MB[row,col] - MA[row,col], for all elements
const	EWISE_SUBR	= 11	' MA[row,col] = MA[row,col] + MB[row,col], for all elements
const	EWISE_MUL	= 12	' MA[row,col] = MA[row,col] * MB[row,col], for all elements
const	EWISE_DIV	= 13	' MA[row,col] = MA[row,col] / MB[row,col], for all elements
const	EWISE_DIVR	= 14	' MA[row,col] = MB[row,col] / MA[row,col], for all elements
const	EWISE_POW	= 15	' MA[row,col] = MA[row,col] ** MB[row,col], for all elements
const	MX_MULTIPLY	= 16	' MA = MB * MC matrix multiply
const	MX_IDENTITY	= 17	' MA = identity matrix
const	MX_DIAGONAL	= 18	' MA = diagonal matrix
const	MX_TRANSPOSE= 19	' MA = transpose of MB
const	MX_COUNT	= 20	' reg[0] = number of elements in MA
const	MX_SUM		= 21	' reg[0] = sum of all elements in MA
const	MX_AVE		= 22	' reg[0] = average value of all elements in MA
const	MX_MIN		= 23	' reg[0] = minimum value of all elements in MA
const	MX_MAX		= 24	' reg[0] = maximum value of all elements in MA
const	MX_COPYAB	= 25	' copy MA to MC
const	MX_COPYAC	= 26	' copy MA to MC
const	MX_COPYBA	= 27	' copy MB to MA
const	MX_COPYBC	= 28	' copy MB to MC
const	MX_COPYCA	= 29	' copy MC to MA
const	MX_COPYCB	= 30	' copy MC to MB
const	MX_DETERM	= 31	' reg0 = determinant of MA (2x2 and 3x3)
const	MX_INVERSE	= 32	' MA = inverse of MB (2x2 and 3x3)
const	MX_ILOADRA	= 33	' indexed load registers to MA
const	MX_ILOADRB	= 34	' indexed load registers to MB
const	MX_ILOADRC	= 35	' indexed load registers to MC
const	MX_ILOADBA	= 36	' indexed load MB to MA
const	MX_ILOADCA	= 37	' indexed load MC to MA
const	MX_ISAVEAR	= 38	' indexed save MA to registers
const	MX_ISAVEAB	= 39	' indexed save MA to MB
const	MX_ISAVEAC	= 40	' indexed save MA to MC

'-------------------- uM-FPU V3.1 FFT definitions -----------------------------

const	FIRST_STAGE	= 0		' first stage of FFT
const	NEXT_STAGE	= 1		' next stage of FFT
const	NEXT_LEVEL	= 2		' next level of FFT
const	NEXT_BLOCK	= 3		' next block of FFT
const	BIT_REVERSE	= 4		' bit reverse sort (pre-processing)
const	PRE_ADJUST	= 8		' inverse FFT (pre-processing)
const	POST_ADJUST	= 16	' inverse FFT (post-processing)

#endif