'==============================================================================
' Copyright  Micromega Corporation 2002-2007. All rights reserved.
'
' @file: 	demo1.bas
' @target:	ARMmite/ARMexpress
'
' This program provides an example of using the uM-FPU V3 floating point
' coprocessor with the ARMmite/ARMexpress using an I2C interface. It takes
' temperature readings from a DS1620 digital thermometer, converts them to
' floating point and displays them in degrees Celsius and degrees Fahrenheit.
'
' @author Cam Thompson, Micromega Corporation
' @version
'	August 2, 2007
'	- updated for uM-FPU V3.1
'	November 9, 2006
'	- original version
'==============================================================================

'-------------------- uM-FPU V3 definitions -----------------------------------

#include "FPUi2c.bas"

'-------------------- DS1620 pin definitions ----------------------------------

const	DS_RST		= 12				' DS1620 reset/enable
const	DS_CLK      = 11				' DS1620 clock
const	DS_DATA     = 10				' DS1620 data

'-------------------- uM-FPU register definitions -----------------------------

const	DegC        = 8					' degrees Celsius
const	DegF        = 9					' degrees Fahrenheit
const	F1_8        = 10				' constant 1.8

'==============================================================================
'-------------------- main routine --------------------------------------------
'==============================================================================

Main:
	print
	print "Demo 1"
	print "------"

	gosub Fpu_Reset						' reset the uM-FPU V3 chip
	if status <> SYNC_CHAR then			' check for synchronization
		print "uM-FPU not detected"
		end
	else
		gosub Print_Version				' display the uM-FPU version number
		print
	endif

	gosub Init_DS1620					' initialize DS1620

										' load floating point constant 1.8
	I2COUT FpuPin, FpuID, [0, SELECTA, F1_8, ATOF, "1.8", 0, FSET0]

'-------------------- main loop -----------------------------------------------

	do
		gosub Read_DS1620				' get temperature reading from DS1620
		print "Raw Temp:   $"; HEX(rawTemp)

										' send rawTemp to uM-FPU
										' convert to floating point
										' store in register
										' divide by 2 to get degrees Celsius
		I2COUT FpuPin, FpuID, [0, SELECTA, DegC, LOADWORD, rawTemp>>8, rawTemp, FSET0, FDIVI, 2]

										' degF = degC * 1.8 + 32
		I2COUT FpuPin, FpuID, [0, SELECTA, DegF, FSET, DegC, FMUL, F1_8, FADDI, 32]

		print "Degrees C: ";			' display degrees Celsius
		I2COUT FpuPin, FpuID, [0, SELECTA, DegC]
		format = 51
		gosub Print_FloatFormat
		print

		print "Degrees F: ";			' display degrees Fahrenheit
		I2COUT FpuPin, FpuID, [0, SELECTA, DegF]
		format = 51
		gosub Print_FloatFormat
		print

		WAIT(2000)						' delay 2 seconds
		print
	loop

'-------------------- Init_DS1620 ---------------------------------------------

Init_DS1620:
	IO(DS_RST) = 0						' initialize pin states
	IO(DS_CLK) = 1
	WAIT(100)

	OUT(DS_RST) = 1						' configure for CPU control
	SHIFTOUT DS_DATA, DS_CLK, 0, [$0C, $02]
	OUT(DS_RST) = 0
	WAIT(100)

	OUT(DS_RST) = 1						' start temperature conversions
	SHIFTOUT DS_DATA, DS_CLK, 0, [$EE]
	OUT(DS_RST) = 0
	WAIT(1000)							' wait for first conversion
	return

'-------------------- Read_DS1620 ---------------------------------------------

Read_DS1620:
	OUT(DS_RST) = 1						' read temperature value
	SHIFTOUT DS_DATA, DS_CLK, 0, [$AA]
	SHIFTIN  DS_DATA, DS_CLK, 1, [rawTemp\9]
	OUT(DS_RST) = 0						' extend the sign bit
	if (rawTemp AND 0x100) then rawTemp = rawTemp OR 0xFFFFFF00
	return
