'==============================================================================
' Copyright  Micromega Corporation 2002-2007. All rights reserved.
'
' @file:	sample.bas
' @target:	ARMmite/ARMexpress
'
' This program provides an example of using the uM-FPU V3 floating point
' coprocessor with the ARMmite/ARMexpress using an I2C interface. This demo
' takes an integer value representing the diameter of a circle in centimeters,
' converts the value to inches, then calculates the circumference and area in
' inches and square inches.
'
' @author Cam Thompson, Micromega Corporation
' @version
'	August 2, 2007
'	- updated for uM-FPU V3.1
'	November 9, 2006
'	- original version
'==============================================================================

'-------------------- uM-FPU V3 definitions -----------------------------------

#include "FPUi2c.bas"

'-------------------- uM-FPU register definitions -----------------------------

const	DiameterIn		= 1				' dimeter in inches
const	Circumference	= 2				' circumference
const	Area			= 3				' area
const	Pi				= 4				' constant pi

'==============================================================================
'-------------------- main routine --------------------------------------------
'==============================================================================

Main:
	print
	print "Conversion Example"
	print "------------------"

	'------------------------------------
	' Reset the uM-FPU and print version.
	'------------------------------------
	gosub Fpu_Reset						' reset the uM-FPU V3 chip
	if status <> SYNC_CHAR then			' check for synchronization
		print "uM-FPU not detected"
		end
	else
		gosub Print_Version				' display the uM-FPU version number
		print
	endif

	'-----------------------------
	' Load constant for later use.
	'-----------------------------
	I2COUT FpuPin, FpuID, [0, SELECTA, Pi, LOADPI, FSET0]

	'--------------------------------------------------------------------------
	' Get diameter in centimeters.  The value would typically come from a sensor
	' reading, but in this example an assumed value of 25 is used.
	'----------------------------------------------------------------------------
	diameterCm = 25
	print "Diameter (cm):       "; diameterCm
	
	'------------------------------
	' convert inches to centimeters
	'------------------------------
	I2COUT FpuPin, FpuID, [0, SELECTA, DiameterIn, FSETI, diameterCm, FCNV, 5]
	print "Diameter (in.):      ";
	gosub Print_Float
	print
	
	'------------------------------
	' circumference = diameter * pi
	'------------------------------
	I2COUT FpuPin, FpuID, [0, SELECTA, Circumference, FSET, DiameterIn, FMUL, Pi]
	print "Circumference (in.): ";
	gosub Print_Float
	print
	
	'----------------------------
	' area = (diameter / 2)^2 * pi
 	'----------------------------
	I2COUT FpuPin, FpuID, [0, SELECTA, Area, FSET, DiameterIn, FDIVI, 2, FMUL, Area, FMUL, Pi]
	print "Area (sq.in.):       ";
	gosub Print_Float
	print
	
	print "Done."
	end
