'==============================================================================
' Copyright  Micromega Corporation 2002-2007. All rights reserved.
'
' @file: 	sensirion.bas
' @target:	ARMmite/ARMexpress
'
' This program provides an example of using the uM-FPU V3 floating point
' coprocessor with the ARMmite/ARMexpress using an I2C interface. It converts
' temperature and humidity readings from a Sensirion SHT1x sensor.
'
' @author Cam Thompson, Micromega Corporation
' @version
'	August 2, 2007
'	- updated for uM-FPU V3.1
'	November 9, 2006
'	- original version
'==============================================================================

'-------------------- uM-FPU V3 definitions -----------------------------------

#include "FPUi2c.bas"

'-------------------- Sensirion SHT1x/SHT7x definitions------------------------

const	ShtClock	= 8					' Sensirion SHT1x clock
const	ShtData		= 9					' Sensirion SHT1x data

'-------------------- uM-FPU register definitions -----------------------------

const	RawTemp		= 10				' raw temperature
const	TempC		= 11				' temperature - Celsius
const	TempF		= 12				' temperature - Fahrenheit
const	RawHumidity	= 13				' raw humidity
const	RhLinear	= 14				' relative humidity - linear
const	RhTrue		= 15				' relative humidity - compensated
const	H			= 16				' H value for dew point calculation
const	DewPoint	= 17				' dew point

const	C2			= 20				' constants for RhLinear calculation
const	C3			= 21
const	T1			= 22				' constants for RhTrue calculation
const	T2			= 23
const	K1			= 24				' constant for DewPoint calculation 
const	K2			= 25
const	K3			= 26
const	K4			= 27

'==============================================================================
'-------------------- main routine --------------------------------------------
'==============================================================================

Main:
	print
	print "Sensirion Demo"
	
	gosub Fpu_Reset						' reset the uM-FPU V3 chip
	if status <> SYNC_CHAR then			' check for synchronization
		print "uM-FPU not detected"
		end
	else
		gosub Print_Version				' display the uM-FPU version number
		print
	endif
	
	gosub SHT_Reset						' reset the Sensirion sensor
	
										' store floating point constants
	I2COUT FpuPin, FpuID, [0, SELECTA, C2, ATOF, "0.0405", 0, FSET0]
	I2COUT FpuPin, FpuID, [0, SELECTA, C3, ATOF, "-2.8e-6", 0, FSET0]
	I2COUT FpuPin, FpuID, [0, SELECTA, T1, ATOF, "0.01", 0, FSET0]
	I2COUT FpuPin, FpuID, [0, SELECTA, T2, ATOF, "0.00008", 0, FSET0]
	I2COUT FpuPin, FpuID, [0, SELECTA, K1, ATOF, "0.4343", 0, FSET0]
	I2COUT FpuPin, FpuID, [0, SELECTA, K2, ATOF, "17.62", 0, FSET0]
	I2COUT FpuPin, FpuID, [0, SELECTA, K3, ATOF, "243.12", 0, FSET0]

'-------------------- main loop -----------------------------------------------

	do	
		' read temperature
		'-----------------
		gosub SHT_Temperature
		print "Raw Temperature:  ";
		format = 60
		gosub Print_FloatFormat
		print

		' TempC = -40.0 + 0.01 * rawTemp
		' TempC = RawTemp / 100.0 - 40.0
		'-------------------------------	
		I2COUT FpuPin, FpuID, [0, SELECTA, TempC, FSET, RawTemp, FDIVI, 100, FSUBI, 40]
		print "Degrees C:        ";
		format = 62
		gosub Print_FloatFormat
		print

		' TempF = TempC converted to Fahrenheit
		'--------------------------------------
		I2COUT FpuPin, FpuID, [0, SELECTA, TempF, FSET, TempC, FCNV, 1]
		print "Degrees F:        ";
		format = 62
		gosub Print_FloatFormat
		print

		' read humidity
		'--------------
		gosub SHT_Humidity
		print "Raw Humidity:     ";
		format = 60
		gosub Print_FloatFormat
		print

		' RhLinear = -4.0 + 0.0405 * rawHumidity + (-2.8E-6) * rawHumidity^2
		' RhLinear = ((rawHumidity * C3) + C2) * rawHumidity) - 4
		'-------------------------------------------------------------------
		I2COUT FpuPin, FpuID, [0, SELECTA, RhLinear, FSET, RawHumidity, FMUL, C3]
		I2COUT FpuPin, FpuID, [0, FADD, C2, FMUL, rawHumidity, FSUBI, 4]	
		print "Humidity (linear):";
		format = 62
		gosub Print_FloatFormat
		print

		' RhTrue = (TempC - 25.0) * (0.01 + 0.00008 * RawHumidity) + RhLinear
		' RhTrue = (TempC - 25.0) * (T2 * RawHumidity + T1) + RhLinear
		'--------------------------------------------------------------------	
		I2COUT FpuPin, FpuID, [0, SELECTA, RhTrue, FSET, TempC, FSUBI, 25]
		I2COUT FpuPin, FpuID, [0, F_LEFT, FSET, T2, FMUL, RawHumidity, FADD, T1]	
		I2COUT FpuPin, FpuID, [0, F_RIGHT, FMUL0, FADD, RhLinear]
		print "Humidity (true):  ";
		format = 62
		gosub Print_FloatFormat
		print

		' H = (log10(RhTrue) - 2.0) / 0.4343 + (17.62 * TempC) / (243.12 + TempC)
		' H = (log10(RhTrue) - 2.0) / K1 + (K2 * TempC) / (K3 + TempC)
		'------------------------------------------------------------------------
		I2COUT FpuPin, FpuID, [0, SELECTA, H, FSET, RhTrue, LOG10, FSUBI, 2, FDIV, K1]	
		I2COUT FpuPin, FpuID, [0, F_LEFT, FSET, K2, FMUL, TempC, F_LEFT, FSET, K3, FADD, TempC]
		I2COUT FpuPin, FpuID, [0, F_RIGHT, FDIV0, F_RIGHT, FADD0]

		' DewPoint = 243.12 * H / (17.62 - H)
		' DewPoint = K3 * H / (K2 - H)
		'------------------------------------------------------------------------
		I2COUT FpuPin, FpuID, [0, SELECTA, DewPoint, FSET, K3, FMUL, H]
		I2COUT FpuPin, FpuID, [0, F_LEFT, FSET, K2, FSUB, H, F_RIGHT, FDIV0]
		print "Dew Point:        ";
		format = 62
		gosub Print_FloatFormat
		print

		WAIT(2000)
		print
	loop

'-------------------- Sensirion SHT1x/SHT7x routines --------------------------

SHT_Reset:								' reset the Sensirion SHT1x
	SHIFTOUT ShtData, ShtClock, 1, [$1FF\9]
	return

SHT_Temperature:						' read temperature and send to uM-FPU
	for param2 = 1 to 100
		param1 = $03
		gosub SHT_Write
		if NOT result then exit
		gosub SHT_Reset					' if NAK, reset the device and retry
	next
	gosub SHT_Wait
	gosub SHT_ReadWord
	I2COUT FpuPin, FpuID, [0, SELECTA, RawTemp, LOADWORD, result>>8, result, FSET0]
	return

SHT_Humidity:							' read humidity and send to uM-FPU
	for param2 = 1 to 100
		param1 = $05
		gosub SHT_Write
		if NOT result then exit
		gosub SHT_Reset					' if NAK, reset the device and retry
	next
	gosub SHT_Wait
	gosub SHT_ReadWord
	I2COUT FpuPin, FpuID, [0, SELECTA, RawHumidity, LOADWORD, result>>8, result, FSET0]
	return

SHT_ReadStatus:							' read status byte
	param1 = $07
	gosub SHT_Write
	gosub SHT_Read
	return

SHT_WriteStatus:						' write status byte
	param2 = param1
	param1 = $06
	gosub SHT_Write
	param1 = param2
	gosub SHT_Write
	return

SHT_Write:
	INPUT ShtData						' send start sequence
	IO(ShtClock) = 0
	OUT(ShtClock) = 1
	IO(ShtData) = 0
	OUT(ShtClock) = 0
	OUT(ShtClock) = 1
	INPUT ShtData
	OUT(ShtClock) = 0
										' send command and get ACK
	SHIFTOUT ShtData, ShtClock, 1, [param1]
	result = IO(ShtData)
	OUT(ShtClock) = 1
	OUT(ShtClock) = 0
	return

SHT_Read:								' read byte and send ACK
	SHIFTIN  ShtData, ShtClock, MSBPRE, [result]
	IO(ShtData) = param1
	OUT(ShtClock) = 1
	OUT(ShtClock) = 0
	INPUT ShtData
	return

SHT_ReadWord:							' read 16-bit value
	param1 = 0
	gosub SHT_Read
	param2 = result << 8
	param1 = 1
	gosub SHT_Read
	result = param2 OR result
	return
  
SHT_Wait:								' wait for ready (~250 msec timeout)
	for param1 = 1 to 250
		if NOT in(ShtData) then exit
		WAIT(1)
	next
	return
