#ifndef FPUspi
#define FPUspi
#include "FPUdefs.bas"

'==============================================================================
' Copyright  Micromega Corporation 2002-2007. All rights reserved.
'
' @file		FPUspi.bas
' @target:	ARMmite/ARMexpress
'
' This file contains support code for using the uM-FPU V3 floating point
' coprocessor with the ARMexpress or ARMmite over an SPI interface.
'
' @author Cam Thompson, Micromega Corporation
' @version
'   August 13, 2007
'   - use #define for FpuCS definition
'	August 2, 2007
'	- original version
'==============================================================================

'==============================================================================
'-------------------- uM-FPU V3 SPI definitions -------------------- 2007-08-02
'==============================================================================

const	FpuClk		= 15		' SPI clock (connects to SCLK/SCL)
const	FpuOut		= 14		' SPI data out (connects to SIN/SDA)
const	FpuIn		= 13		' SPI data in  (connects to SOUT) Note: set to
								' same pin number as FpuOut for 2-wire SPI
#define	FpuCS		-1			' SPI CS (connects to CS) Note: set to -1
								' unless the CS pin has been enabled on FPU

'==============================================================================
'-------------------- uM-FPU V3 SPI support routines --------------- 2007-08-02
'==============================================================================

Fpu_Reset:
	SPIMODE 0
	SPIOUT FpuCS, FpuOut, FpuClk, [$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,0]
	WAIT(10)

	SPIOUT FpuCS, FpuOut, FpuClk, [SYNC] ' check for synchronization
	goto Fpu_Status2

Fpu_Wait:
  #if FpuCS >= 0
	OUT(FpuCS) = 0
  #endif
	INPUT(FpuIn)						' wait until uM-FPU is ready
	while (IN(FpuIn))
	loop
  #if FpuCS >= 0
	OUT(FpuCS) = 1
  #endif
	return

Fpu_ReadStatus:
	gosub Fpu_Wait						' read status byte
	SPIOUT FpuCS, FpuOut, FpuClk, [READSTATUS]

Fpu_Status2:
	gosub Fpu_ReadDelay
	SPIIN FpuCS, FpuIn, FpuClk, [status]
	return

Fpu_ReadDelay:
	time1 = TIMER
	while ((TIMER - time1) < 15)		' read setup delay of 15 usec
	loop
	return

Fpu_ReadString:
	gosub Fpu_Wait						' wait until uM-FPU is ready
	SPIOUT FpuCS, FpuOut, FpuClk, [READSTR]	
	gosub FPU_ReadDelay
	SPIIN FpuCS, FpuIn, FpuClk, [str$]	' read string from uM-FPU
	return

Fpu_WriteFloat:							' write 32-bit floating point
	if reg = -1 then
		SPIOUT FpuCS, FpuOut, FpuClk, [FWRITEA]
	elseif reg = -2 then
		SPIOUT FpuCS, FpuOut, FpuClk, [FWRITEX]
	else
		SPIOUT FpuCS, FpuOut, FpuClk, [FWRITE, reg]
	endif
	goto Fpu_Write32

Fpu_WriteLong:							' write 32-bit integer
	if reg = -1 then
		SPIOUT FpuCS, FpuOut, FpuClk, [LWRITEA]
	elseif reg = -2 then
		SPIOUT FpuCS, FpuOut, FpuClk, [LWRITEX]
	else
		SPIOUT FpuCS, FpuOut, FpuClk, [LWRITE, reg]
	endif
										' (fall through to Fpu_Write32)
Fpu_Write32:
	SPIOUT FpuCS, FpuOut, FpuClk, [tmp>>24, tmp>>16, tmp>>8, tmp]
	return

Fpu_ReadFloat:							' read 32-bit floating point
	gosub Fpu_Wait
	if reg = -1 then
		SPIOUT FpuCS, FpuOut, FpuClk, [LREADA]
	elseif reg = -2 then
		SPIOUT FpuCS, FpuOut, FpuClk, [LREADX]
	else
		SPIOUT FpuCS, FpuOut, FpuClk, [LREAD, reg]
	endif
	goto Fpu_Read32
	
Fpu_ReadLong:							' read 32-bit integer
	gosub Fpu_Wait
	if reg = -1 then
		SPIOUT FpuCS, FpuOut, FpuClk, [LREADA]
	elseif reg = -2 then
		SPIOUT FpuCS, FpuOut, FpuClk, [LREADX]
	else
		SPIOUT FpuCS, FpuOut, FpuClk, [LREAD, reg]
	endif
										' (fall through to Fpu_Read32)	
Fpu_Read32:
	gosub Fpu_ReadDelay
	SPIIN FpuCS, FpuIn, FpuClk, [tmp, tmp2, tmp3, tmp4]
	tmp = (tmp<<24) + (tmp2<<16) + (tmp3<<8) + tmp4
	return

Fpu_Fcall:
	SPIOUT FpuCS, FpuOut, FpuClk, [FCALL, tmp]
	return
	
Print_Version:							' print the uM-FPU version string
	SPIOUT FpuCS, FpuOut, FpuClk, [VERSION]
	goto Print_FpuString

Print_Float:
	format = 0							' set format to zero (free format)
										' (fall through to Print_FloatFormat)
Print_FloatFormat:
	opcode = FTOA						' convert floating point to formatted ASCII
	goto Print_Format

Print_Long:
	format = 0							' set format to 0 (free format)
										' (fall through to Print_LongFormat)
Print_LongFormat:
	opcode = LTOA						' convert long integer to formatted ASCII
										' (fall through to Print_Format)

Print_Format:							' send conversion command
	SPIOUT FpuCS, FpuOut, FpuClk, [opcode, format]

Print_FpuString:
	gosub Fpu_ReadString				' read string from uM-FPU and print
	print str$;
	return

#endif