'==============================================================================
' Copyright  Micromega Corporation 2002-2007. All rights reserved.
'
' @file:	graph.bas
' @target:	ARMmite/ARMexpress
'
' This program provides an example of using the uM-FPU V3 floating point
' coprocessor with the ARMmite/ARMexpress using an SPI interface. It draws a
' simple graph of the sine and cosine functions in the terminal window.
'
' @author Cam Thompson, Micromega Corporation
' @version
'	August 2, 2007
'	- updated for uM-FPU V3.1
'	November 9, 2006
'	- original version
'===============================================================================

'-------------------- uM-FPU V3 definitions -----------------------------------

#include "FPUspi.bas"

'-------------------- uM-FPU register definitions -----------------------------

const	Angle		= 1					' angle
const	Point		= 2					' point on graph

'-------------------- constants -----------------------------------------------

const	Space		= 32				' ASCII character values
const	Asterisk	= 42
const	VerticalLine= 124

'==============================================================================
'-------------------- main routine --------------------------------------------
'==============================================================================

Main:
	print
	print "Graph demo"
	
	gosub Fpu_Reset						' reset the uM-FPU V3 chip
	if status <> SYNC_CHAR then			' check for synchronization
		print "uM-FPU not detected"
		end
	else
		gosub Print_Version				' display the uM-FPU version number
		print
	endif

	' graph sine from 0 TO 360 degrees (2*PI radians)
	'------------------------------------------------
	print
	print "Graph of sine"
	print "-1        0         1"
	print "---------------------"

	for cnt = 0 TO 20
		' Angle = sin(pi * 2 * cnt / 20)
		'-------------------------------
		gosub Fpu_Wait
		SPIOUT FpuCS, FpuOut, FpuClk, [SELECTA, Angle, LOADPI, FSET0]
		SPIOUT FpuCS, FpuOut, FpuClk, [FMULI, cnt*2, FDIVI, 20, F_SIN]
		gosub graph						' graph the result
	next

	' graph cosine from 0 TO 360 degrees (2*PI radians)
	'--------------------------------------------------
	print
	print "Graph of cosine"
	print "-1        0         1"
	print "---------------------"
	
	for cnt = 0 TO 20
		' Angle = cos(pi * 2 * cnt / 20)
		'-------------------------------
		gosub Fpu_Wait
		SPIOUT FpuCS, FpuOut, FpuClk, [SELECTA, Angle, LOADPI, FSET0]
		SPIOUT FpuCS, FpuOut, FpuClk, [FMULI, cnt*2, FDIVI, 20, F_COS]
		gosub graph						' graph the result
	next

Done:
	print "Done."
	end

'-------------------- Graph ---------------------------------------------------

Graph:
	' Point = Fix(Round(Angle * 10))
	'-------------------------------
	gosub Fpu_Wait
	SPIOUT FpuCS, FpuOut, FpuClk, [SELECTA, Point, FSET, Angle, FMULI, 10, FIXR]

	'Read the lower 8 bits of long value
	'-----------------------------------
	gosub Fpu_Wait
	SPIOUT FpuCS, FpuOut, FpuClk, [LREADBYTE]
	gosub Fpu_ReadDelay
	SPIIN FpuCS, FpuIn, FpuClk, [p]
	if (p AND $80) then p = p OR $FFFFFF00

	' add offset to midpoint of graph
	'--------------------------------
	p = p + 10

	'plot the point on graph
	'-----------------------
	for x = 0 TO 20
		if (x = p) then
			tmp = Asterisk
		elseIF (x = 10) then
			tmp = VerticalLine
		else
			tmp = Space
		endif
		print CHR(tmp);
	next

	'display the original angle in 8.5 format
	'----------------------------------------
	print " ";
	SPIOUT FpuCS, FpuOut, FpuClk, [SELECTA, Angle]
	format = 85							' (8.5 format i.e. width=8, decimals=5)
	gosub Print_FloatFormat
	print
	return
